"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../logger");
const getWorkspace_1 = require("../workspace/getWorkspace");
const generateTopologicalGraph_1 = require("../workspace/generateTopologicalGraph");
const task_scheduler_1 = require("@microsoft/task-scheduler");
const getPipelinePackages_1 = require("../task/getPipelinePackages");
const parsePipelineConfig_1 = require("../task/parsePipelineConfig");
const path_1 = __importDefault(require("path"));
const getNpmCommand_1 = require("../task/getNpmCommand");
/**
 * Generates a graph and spit it out in stdout
 *
 * Expected format:
 * [
 *   {
 *       "id": "bar##build",
 *       "package": "bar",
 *       "task": "build",
 *       "command": "npm run build --blah",
 *       "workingDirectory": "packages/bar",
 *       "dependencies": []
 *   },
 *   {
 *       "id": "foo##build",
 *       "package": "foo",
 *       "task": "build",
 *       "command": "npm run build --blah",
 *       "workingDirectory": "packages/foo",
 *       "dependencies": [
 *           "bar##build"
 *       ]
 *   },
 *   {
 *       "id": "foo##test",
 *       "package": "foo",
 *       "task": "test",
 *       "command": "npm run test --blah",
 *       "workingDirectory": "packages/foo",
 *       "dependencies": [
 *           "foo##build"
 *       ]
 *   },
 *   ...
 * ]
 */
async function info(cwd, config) {
    const workspace = getWorkspace_1.getWorkspace(cwd, config);
    const tasks = new Map();
    const pipelineConfig = parsePipelineConfig_1.parsePipelineConfig(config.pipeline);
    for (const [taskName, taskDeps] of Object.entries(pipelineConfig.taskDeps)) {
        const { deps, topoDeps } = taskDeps;
        tasks.set(taskName, {
            name: taskName,
            run: () => Promise.resolve(true),
            deps,
            topoDeps,
        });
    }
    const graph = generateTopologicalGraph_1.generateTopologicGraph(workspace);
    const packages = getPipelinePackages_1.getPipelinePackages(workspace, config);
    const taskDeps = task_scheduler_1.generateTaskGraph(packages, config.command.slice(1), tasks, graph, pipelineConfig.packageTaskDeps, false);
    const packageTasks = new Map();
    for (const [fromId, toId] of taskDeps) {
        let fromPackageTask = packageTasks.get(fromId);
        let toPackageTask = packageTasks.get(toId);
        // Try creating missing package task info
        if (!fromPackageTask) {
            fromPackageTask = createPackageTaskInfo(fromId, config, workspace);
            if (fromPackageTask) {
                packageTasks.set(fromId, fromPackageTask);
            }
        }
        if (!toPackageTask) {
            toPackageTask = createPackageTaskInfo(toId, config, workspace);
            if (toPackageTask) {
                packageTasks.set(toId, toPackageTask);
            }
        }
        // If "from" AND "to" package tasks are valid, then connect them
        if (fromPackageTask && toPackageTask) {
            toPackageTask.dependencies.push(fromId);
        }
    }
    logger_1.logger.info(`info`, {
        command: config.command.slice(1),
        scope: packages,
        packageTasks: [...packageTasks.values()],
    });
}
exports.info = info;
function createPackageTaskInfo(taskId, config, workspace) {
    const [pkg, taskName] = task_scheduler_1.getPackageTaskFromId(taskId);
    const scripts = workspace.allPackages[pkg].scripts;
    if (scripts && scripts[taskName]) {
        return {
            id: taskId,
            command: [
                config.npmClient,
                ...getNpmCommand_1.getNpmCommand(config.node, config.args, taskName),
            ],
            dependencies: [],
            workingDirectory: path_1.default
                .relative(workspace.root, path_1.default.dirname(workspace.allPackages[pkg].packageJsonPath))
                .replace(/\\/g, "/"),
            package: pkg,
            task: taskName,
        };
    }
}
