"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getWorkspace_1 = require("../workspace/getWorkspace");
const logger_1 = require("../logger");
const generateTopologicalGraph_1 = require("../workspace/generateTopologicalGraph");
const abortSignal_1 = require("../task/abortSignal");
const displayReportAndExit_1 = require("../displayReportAndExit");
const context_1 = require("../context");
const taskRunner_1 = require("../task/taskRunner");
const NpmScriptTask_1 = require("../task/NpmScriptTask");
const task_scheduler_1 = require("@microsoft/task-scheduler");
const getPipelinePackages_1 = require("../task/getPipelinePackages");
const parsePipelineConfig_1 = require("../task/parsePipelineConfig");
// Run multiple
async function run(cwd, config, reporters) {
    const context = context_1.createContext(config);
    const workspace = getWorkspace_1.getWorkspace(cwd, config);
    const tasks = new Map();
    const pipelineConfig = parsePipelineConfig_1.parsePipelineConfig(config.pipeline);
    for (const [taskName, taskDeps] of Object.entries(pipelineConfig.taskDeps)) {
        const { deps, topoDeps } = taskDeps;
        tasks.set(taskName, {
            name: taskName,
            run: () => Promise.resolve(true),
            deps,
            topoDeps,
        });
    }
    // generate topological graph
    const graph = generateTopologicalGraph_1.generateTopologicGraph(workspace);
    const packages = getPipelinePackages_1.getPipelinePackages(workspace, config);
    const taskDeps = task_scheduler_1.generateTaskGraph(packages, config.command, tasks, graph, pipelineConfig.packageTaskDeps, false);
    const { profiler } = context;
    let aborted = false;
    context.measures.start = process.hrtime();
    // die faster if an abort signal is seen
    abortSignal_1.signal.addEventListener("abort", () => {
        aborted = true;
        NpmScriptTask_1.NpmScriptTask.killAllActiveProcesses();
        displayReportAndExit_1.displayReportAndExit(reporters, context);
    });
    try {
        await taskRunner_1.runTasks({ graph, workspace, context, config, packageTaskDeps: taskDeps });
    }
    catch (e) {
        logger_1.logger.error("runTasks: " + (e.stack || e.message || e));
        process.exitCode = 1;
    }
    if (config.profile) {
        const profileFile = profiler.output();
        logger_1.logger.info(`runTasks: Profile saved to ${profileFile}`);
    }
    if (!aborted) {
        displayReportAndExit_1.displayReportAndExit(reporters, context);
    }
}
exports.run = run;
