"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const getWorkspace_1 = require("../workspace/getWorkspace");
const workerQueue_1 = require("../task/workerQueue");
const child_process_1 = require("child_process");
const path = __importStar(require("path"));
const findNpmClient_1 = require("../workspace/findNpmClient");
const TaskLogWritable_1 = require("../logger/TaskLogWritable");
const TaskLogger_1 = require("../logger/TaskLogger");
const backfill_1 = require("../cache/backfill");
const task_scheduler_1 = require("@microsoft/task-scheduler");
// Run multiple
async function worker(cwd, config, reporters) {
    const workspace = getWorkspace_1.getWorkspace(cwd, config);
    const workerQueue = await workerQueue_1.initWorkerQueue(config.workerQueueOptions);
    workerQueue.on('job succeeded', async () => {
        const jobs = await workerQueue.getJobs('waiting', { size: 1 });
        if (jobs.length === 0) {
            workerQueue.close();
        }
    });
    workerQueue.process(config.concurrency, async (job, done) => {
        console.log(`processing job ${job.id}: ${job.data.name} ${job.data.task}`);
        await Promise.all(job.data.taskDeps.map((taskDep) => {
            const [name, task] = taskDep.split("#");
            if (task) {
                const info = workspace.allPackages[name];
                const packagePath = path.dirname(info.packageJsonPath);
                return getCache({
                    task,
                    name,
                    root: cwd,
                    packagePath,
                    config,
                });
            }
        }));
        const cacheResult = await getCache({
            task: job.data.task,
            name: job.data.name,
            root: cwd,
            packagePath: path.join(cwd, job.data.packagePath),
            config,
        });
        if (cacheResult.cacheHit) {
            return done();
        }
        const { npmArgs, spawnOptions, packagePath } = job.data;
        const npmCmd = findNpmClient_1.findNpmClient(config.npmClient);
        const cp = child_process_1.spawn(npmCmd, npmArgs, Object.assign({ cwd: path.join(cwd, packagePath) }, spawnOptions));
        const logger = new TaskLogger_1.TaskLogger(job.data.name, job.data.task);
        const stdoutLogger = new TaskLogWritable_1.TaskLogWritable(logger);
        cp.stdout.pipe(stdoutLogger);
        const stderrLogger = new TaskLogWritable_1.TaskLogWritable(logger);
        cp.stderr.pipe(stderrLogger);
        cp.on("exit", async () => {
            await saveCache(cacheResult.hash, {
                task: job.data.task,
                name: job.data.name,
                root: cwd,
                packagePath: job.data.packagePath,
                config,
            });
            done();
        });
    });
}
exports.worker = worker;
// speeding up to reduce network costs for a worker
const localHashCache = {};
async function getCache(options) {
    let hash = null;
    let cacheHit = false;
    const { task, name, root, packagePath, config } = options;
    const localCacheKey = task_scheduler_1.getTaskId(name, task);
    if (localHashCache[localCacheKey]) {
        return { hash: localHashCache[localCacheKey], cacheHit: true };
    }
    if (config.cache) {
        hash = await backfill_1.cacheHash(task, name, root, packagePath, config.cacheOptions, config.passThroughArgs);
        if (hash && !config.resetCache) {
            cacheHit = await backfill_1.cacheFetch(hash, task, name, packagePath, config.cacheOptions);
            if (cacheHit) {
                localHashCache[localCacheKey] = hash;
            }
        }
    }
    return { hash, cacheHit };
}
async function saveCache(hash, options) {
    const { logger, packagePath, config } = options;
    await backfill_1.cachePut(hash, packagePath, config.cacheOptions);
}
