"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const TaskLogger_1 = require("../logger/TaskLogger");
const findNpmClient_1 = require("../workspace/findNpmClient");
const child_process_1 = require("child_process");
const abortSignal_1 = require("./abortSignal");
const path_1 = __importDefault(require("path"));
const TaskLogWritable_1 = require("../logger/TaskLogWritable");
const backfill_1 = require("../cache/backfill");
const formatDuration_1 = require("../logger/reporters/formatDuration");
const getNpmCommand_1 = require("./getNpmCommand");
class NpmScriptTask {
    constructor(task, root, info, config, context) {
        this.task = task;
        this.root = root;
        this.info = info;
        this.config = config;
        this.context = context;
        this.npmArgs = [];
        this.startTime = [0, 0];
        this.duration = [0, 0];
        NpmScriptTask.npmCmd =
            NpmScriptTask.npmCmd || findNpmClient_1.findNpmClient(config.npmClient);
        this.status = "pending";
        this.logger = new TaskLogger_1.TaskLogger(info.name, task);
        this.npmArgs = getNpmCommand_1.getNpmCommand(config.nodeArgs, config.passThroughArgs, task);
    }
    static killAllActiveProcesses() {
        // first, send SIGTERM everywhere
        for (const cp of NpmScriptTask.activeProcesses) {
            cp.kill("SIGTERM");
        }
        // wait for "gracefulKillTimeout" to make sure everything is terminated via SIGKILL
        setTimeout(() => {
            for (const cp of NpmScriptTask.activeProcesses) {
                if (!cp.killed) {
                    cp.kill("SIGKILL");
                }
            }
        }, NpmScriptTask.gracefulKillTimeout);
    }
    onStart() {
        this.status = "started";
        this.startTime = process.hrtime();
        this.logger.info("started", { status: "started" });
    }
    onComplete() {
        this.status = "completed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("completed", {
            status: "completed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onFail() {
        this.status = "failed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("failed", {
            status: "failed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onSkipped(hash) {
        this.status = "skipped";
        this.duration = process.hrtime(this.startTime);
        this.logger.info(`skipped`, {
            status: "skipped",
            duration: formatDuration_1.hrToSeconds(this.duration),
            hash,
        });
    }
    async getCache() {
        let hash = null;
        let cacheHit = false;
        const { task, info, root, config } = this;
        const name = info.name;
        const packagePath = path_1.default.dirname(info.packageJsonPath);
        if (config.cache) {
            hash = await backfill_1.cacheHash(task, name, root, packagePath, config.cacheOptions, config.passThroughArgs);
            if (hash && !config.resetCache) {
                cacheHit = await backfill_1.cacheFetch(hash, task, name, packagePath, config.cacheOptions);
            }
        }
        return { hash, cacheHit };
    }
    async saveCache(hash) {
        const { logger, info, config, root } = this;
        const packagePath = path_1.default.relative(root, path_1.default.dirname(info.packageJsonPath));
        logger.verbose(`hash put ${hash}`);
        await backfill_1.cachePut(hash, packagePath, config.cacheOptions);
    }
    runScript() {
        const { info, logger, npmArgs } = this;
        const { npmCmd } = NpmScriptTask;
        return new Promise((resolve, reject) => {
            logger.verbose(`Running ${[npmCmd, ...npmArgs].join(" ")}`);
            const cp = child_process_1.spawn(npmCmd, npmArgs, {
                cwd: path_1.default.dirname(info.packageJsonPath),
                stdio: "pipe",
                env: Object.assign(Object.assign(Object.assign({}, process.env), (process.stdout.isTTY &&
                    this.config.reporter !== "json" && { FORCE_COLOR: "1" })), { LAGE_PACKAGE_NAME: info.name }),
            });
            NpmScriptTask.activeProcesses.add(cp);
            const stdoutLogger = new TaskLogWritable_1.TaskLogWritable(this.logger);
            cp.stdout.pipe(stdoutLogger);
            const stderrLogger = new TaskLogWritable_1.TaskLogWritable(this.logger);
            cp.stderr.pipe(stderrLogger);
            cp.on("exit", handleChildProcessExit);
            function handleChildProcessExit(code) {
                if (code === 0) {
                    NpmScriptTask.activeProcesses.delete(cp);
                    return resolve();
                }
                cp.stdout.destroy();
                cp.stdin.destroy();
                reject();
            }
        });
    }
    async run() {
        const { info, task, context, config, logger } = this;
        try {
            const { hash, cacheHit } = await this.getCache();
            const cacheEnabled = config.cache && hash;
            this.onStart();
            // skip if cache hit!
            if (cacheHit) {
                this.onSkipped(hash);
                return true;
            }
            if (cacheEnabled) {
                logger.verbose(`hash: ${hash}, cache hit? ${cacheHit}`);
            }
            await context.profiler.run(() => this.runScript(), `${info.name}.${task}`);
            if (cacheEnabled) {
                await this.saveCache(hash);
            }
            this.onComplete();
        }
        catch (e) {
            context.measures.failedTasks = context.measures.failedTasks || [];
            context.measures.failedTasks.push({ pkg: info.name, task });
            this.onFail();
            if (config.continueOnError) {
                return true;
            }
            if (!config.safeExit) {
                abortSignal_1.controller.abort();
            }
            return false;
        }
        return true;
    }
}
exports.NpmScriptTask = NpmScriptTask;
NpmScriptTask.npmCmd = "";
NpmScriptTask.activeProcesses = new Set();
NpmScriptTask.gracefulKillTimeout = 2500;
