"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const task_scheduler_1 = require("@microsoft/task-scheduler");
const NpmScriptTask_1 = require("./NpmScriptTask");
const getPipelinePackages_1 = require("./getPipelinePackages");
const parsePipelineConfig_1 = require("./parsePipelineConfig");
const DistributedNpmTask_1 = require("./DistributedNpmTask");
const workerQueue_1 = require("./workerQueue");
/** Returns a map that maps task name to the priorities config for that task */
function getPriorityMap(priorities) {
    const result = new Map();
    priorities.forEach((entry) => {
        const taskPriority = result.get(entry.task) || {};
        taskPriority[entry.package] = entry.priority;
        result.set(entry.task, taskPriority);
    });
    return result;
}
async function runTasks(options) {
    const { graph, workspace, context, config, packageTaskDeps } = options;
    if (config.dist) {
        await workerQueue_1.initWorkerQueue(config.workerQueueOptions, false);
    }
    const priorityMap = getPriorityMap(config.priorities);
    let pipeline = task_scheduler_1.createPipeline(graph, {
        // dummy logger for task-scheduler because lage already has logger for its tasks
        logger: {
            error: (_msg) => { },
            log: (_msg) => { },
        },
        exit: (code) => { },
        targetsOnly: config.only,
        concurrency: config.concurrency,
        continue: config.continue,
    });
    const pipelineConfig = parsePipelineConfig_1.parsePipelineConfig(config.pipeline);
    const knownTasks = Object.keys(pipelineConfig.taskDeps);
    let unknownTasks = new Set();
    // Add in the task deps based on package graph
    for (const [taskName, taskDeps] of Object.entries(pipelineConfig.taskDeps)) {
        const { deps, topoDeps } = taskDeps;
        pipeline = pipeline.addTask({
            name: taskName,
            deps,
            topoDeps,
            priorities: priorityMap.get(taskName),
            run: runHandler(workspace, taskName, generateTaskConfig(config), context, config.dist, packageTaskDeps),
        });
        // take note of any tasks deps that are not defined
        const unknownTaskDeps = deps
            .filter((dep) => !knownTasks.includes(dep))
            .concat(topoDeps.filter((dep) => !knownTasks.includes(dep)));
        for (const unknown of unknownTaskDeps) {
            unknownTasks.add(unknown);
        }
    }
    // adding specific package task dependencies
    for (const packageTaskDep of pipelineConfig.packageTaskDeps) {
        const from = task_scheduler_1.getPackageTaskFromId(packageTaskDep[0]);
        const to = task_scheduler_1.getPackageTaskFromId(packageTaskDep[1]);
        pipeline = pipeline.addDep({ package: from[0], task: from[1] }, { package: to[0], task: to[1] });
    }
    // Collect unknown commands
    const unknownCommands = config.command.filter((command) => !knownTasks.includes(command));
    for (const unknown of unknownCommands) {
        unknownTasks.add(unknown);
    }
    // Add all unknown commands to be topoDeps
    for (const taskName of unknownTasks) {
        const depConfig = config.parallel ? { deps: [], topoDeps: [] } : { topoDeps: [taskName], deps: [] };
        pipeline = pipeline.addTask(Object.assign(Object.assign({}, depConfig), { name: taskName, priorities: priorityMap.get(taskName), run: runHandler(workspace, taskName, generateTaskConfig(config), context, config.dist, packageTaskDeps) }));
    }
    await pipeline.go({
        packages: getPipelinePackages_1.getPipelinePackages(workspace, config),
        tasks: config.command,
    });
}
exports.runTasks = runTasks;
function runHandler(workspace, taskName, config, context, distributed, taskDeps) {
    return async function (_location, _stdout, _stderr, pkg) {
        const info = workspace.allPackages[pkg];
        const scripts = info.scripts;
        if (scripts && scripts[taskName]) {
            const npmTask = distributed
                ? new DistributedNpmTask_1.DistributedNpmScriptTask(taskName, workspace.root, info, config, context, taskDeps)
                : new NpmScriptTask_1.NpmScriptTask(taskName, workspace.root, info, config, context);
            context.tasks.set(task_scheduler_1.getTaskId(info.name, taskName), npmTask);
            return await npmTask.run();
        }
        return true;
    };
}
const generateTaskConfig = (config) => ({
    cache: config.cache,
    continueOnError: config.continue,
    safeExit: config.safeExit,
    npmClient: config.npmClient,
    cacheOptions: config.cacheOptions,
    reporter: config.reporter,
    resetCache: config.resetCache,
    nodeArgs: config.node,
    passThroughArgs: config.args,
    workerQueueOptions: config.workerQueueOptions
});
